import type { Plugin } from "obsidian";
import { mount, unmount, type Component, type MountOptions } from "svelte";
import { MarkdownRenderChild } from "obsidian";

export function registerCodeBlockRenderer(
	plugin: Plugin,
	name: string,
	renderer: (source: string, el: HTMLElement) => MarkdownRenderChild
): void {
	plugin.registerMarkdownCodeBlockProcessor(name, (source, el, ctx) => {
		ctx.addChild(renderer(source, el));
	});
}

export class SvelteCodeBlockRenderer<
	TComponent extends Component<TProps, TExports, TBindings>,
	TProps extends Record<string, any> = {},
	TExports extends Record<string, any> = {},
	TBindings extends keyof TProps | "" = string
> extends MarkdownRenderChild {
	protected component: TExports | undefined;

	constructor(
		private readonly contentEl: HTMLElement,
		private readonly componentCtor: TComponent,
		private readonly mountOpts: Omit<MountOptions<TProps>, "target">
	) {
		super(contentEl);
	}

	onload(): void {
		this.component = mount(this.componentCtor, {
			...this.mountOpts,
			target: this.contentEl,
		});
	}

	onunload(): void {
		if (this.component) {
			unmount(this.component);
			this.component = undefined;
		}
	}
}

export { registerReadingLogCodeBlockProcessor } from "./ReadingLogCodeBlock";
